#include <Wire.h>

#define DO_SERIAL

#define BMP280_ADDRESS                  0x76
#define BMP280_REGISTER_CONFIG          0xF5
#define BMP280_REGISTER_CONTROL         0xF4
#define BMP280_REGISTER_DIG_T1          0x88
#define BMP280_REGISTER_DIG_T2          0x8A
#define BMP280_REGISTER_DIG_T3          0x8C
#define BMP280_REGISTER_DIG_P1          0x8E
#define BMP280_REGISTER_DIG_P2          0x90
#define BMP280_REGISTER_DIG_P3          0x92
#define BMP280_REGISTER_DIG_P4          0x94
#define BMP280_REGISTER_DIG_P5          0x96
#define BMP280_REGISTER_DIG_P6          0x98
#define BMP280_REGISTER_DIG_P7          0x9A
#define BMP280_REGISTER_DIG_P8          0x9C
#define BMP280_REGISTER_DIG_P9          0x9E
#define BMP280_REGISTER_PRESSUREDATA    0xF7
#define BMP280_REGISTER_TEMPDATA        0xFA

#ifdef DO_SERIAL
  uint16_t calib_T1;
  int16_t  calib_T2;
  int16_t  calib_T3;

  uint16_t calib_P1;
  int16_t  calib_P2;
  int16_t  calib_P3;
  int16_t  calib_P4;
  int16_t  calib_P5;
  int16_t  calib_P6;
  int16_t  calib_P7;
  int16_t  calib_P8;
  int16_t  calib_P9;

uint16_t read16(byte reg)
{
  uint16_t value;

    Wire.beginTransmission(BMP280_ADDRESS);
    Wire.write((uint8_t)reg);
    Wire.endTransmission();
    Wire.requestFrom(BMP280_ADDRESS, 2);
    value = (Wire.read() << 8) | Wire.read();

  return value;
}

uint16_t read16_LE(byte reg) {
  uint16_t temp = read16(reg);
  return (temp >> 8) | (temp << 8);

}

int16_t readS16(byte reg)
{
  return (int16_t)read16(reg);

}

int16_t readS16_LE(byte reg)
{
  return (int16_t)read16_LE(reg);

}

void readCoefficients(void)
{
    calib_T1 = read16_LE(BMP280_REGISTER_DIG_T1);
    calib_T2 = readS16_LE(BMP280_REGISTER_DIG_T2);
    calib_T3 = readS16_LE(BMP280_REGISTER_DIG_T3);

    calib_P1 = read16_LE(BMP280_REGISTER_DIG_P1);
    calib_P2 = readS16_LE(BMP280_REGISTER_DIG_P2);
    calib_P3 = readS16_LE(BMP280_REGISTER_DIG_P3);
    calib_P4 = readS16_LE(BMP280_REGISTER_DIG_P4);
    calib_P5 = readS16_LE(BMP280_REGISTER_DIG_P5);
    calib_P6 = readS16_LE(BMP280_REGISTER_DIG_P6);
    calib_P7 = readS16_LE(BMP280_REGISTER_DIG_P7);
    calib_P8 = readS16_LE(BMP280_REGISTER_DIG_P8);
    calib_P9 = readS16_LE(BMP280_REGISTER_DIG_P9);

#ifdef DO_SERIAL
  Serial.println( calib_T1 );
  Serial.println( calib_T2 );
  Serial.println( calib_T3 );

  Serial.println( calib_P1 );
  Serial.println( calib_P2 );
  Serial.println( calib_P3 );
  Serial.println( calib_P4 );
  Serial.println( calib_P5 );
  Serial.println( calib_P6 );
  Serial.println( calib_P7 );
  Serial.println( calib_P8 );
  Serial.println( calib_P9 );
#endif
}

int32_t t_fine;
#endif

void write8(byte reg, byte value)
{
    Wire.beginTransmission(BMP280_ADDRESS);
    Wire.write((uint8_t)reg);
    Wire.write((uint8_t)value);
    Wire.endTransmission();  
}

uint32_t read24(byte reg)
{
  uint32_t value;

  Wire.beginTransmission(BMP280_ADDRESS);
  Wire.write((uint8_t)reg);
  Wire.endTransmission();
  Wire.requestFrom(BMP280_ADDRESS, 3);

  value = Wire.read();
  value <<= 8;
  value |= Wire.read();
  value <<= 8;
  value |= Wire.read();

  return value;
}

#ifdef DO_SERIAL
float readTemperature()
{
  int32_t var1, var2;

  int32_t adc_T = read24(BMP280_REGISTER_TEMPDATA);
  adc_T >>= 4;

  var1  = ((((adc_T>>3) - ((int32_t)calib_T1 <<1))) *
     ((int32_t)calib_T2)) >> 11;

  var2  = (((((adc_T>>4) - ((int32_t)calib_T1)) *
       ((adc_T>>4) - ((int32_t)calib_T1))) >> 12) *
     ((int32_t)calib_T3)) >> 14;

  t_fine = var1 + var2;

  float T  = (t_fine * 5 + 128) >> 8;
  return T/100;
}

float readPressure() {
  int64_t var1, var2, p;

  // Must be done first to get the t_fine variable set up
  readTemperature();

  int32_t adc_P = read24(BMP280_REGISTER_PRESSUREDATA);
  adc_P >>= 4;

  var1 = ((int64_t)t_fine) - 128000;
  var2 = var1 * var1 * (int64_t)calib_P6;
  var2 = var2 + ((var1*(int64_t)calib_P5)<<17);
  var2 = var2 + (((int64_t)calib_P4)<<35);
  var1 = ((var1 * var1 * (int64_t)calib_P3)>>8) +
    ((var1 * (int64_t)calib_P2)<<12);
  var1 = (((((int64_t)1)<<47)+var1))*((int64_t)calib_P1)>>33;

  if (var1 == 0) {
    return 0;  // avoid exception caused by division by zero
  }
  p = 1048576 - adc_P;
  p = (((p<<31) - var2)*3125) / var1;
  var1 = (((int64_t)calib_P9) * (p>>13) * (p>>13)) >> 25;
  var2 = (((int64_t)calib_P8) * p) >> 19;

  p = ((p + var1 + var2) >> 8) + (((int64_t)calib_P7)<<4);
  return (float)p/256;
}
#endif

void setup() {
#ifdef DO_SERIAL
  Serial.begin(115200);
#endif

  Wire.begin();
  delay(100);

#ifdef DO_SERIAL
  readCoefficients();
#endif

  write8(BMP280_REGISTER_CONFIG, 0xA0);
  write8(BMP280_REGISTER_CONTROL, 0x6B);
}

void loop() {

  int32_t adc_T = read24(BMP280_REGISTER_TEMPDATA);
  int32_t adc_P = read24(BMP280_REGISTER_PRESSUREDATA);

 #ifdef DO_SERIAL 
    Serial.print(adc_T);
    Serial.print(", ");
    Serial.print(adc_P);
    Serial.print(", ");

    Serial.print(F("Temperature = "));
    Serial.print(readTemperature());
    Serial.print(" *C, ");

    Serial.print(F("Pressure = "));
    Serial.print(readPressure()/100.0f);
    Serial.println(" mbar");
#endif

    delay(500);
}
