
byte GSM_signalQuality;
byte GSM_registered;

//--------------------------------------------------

#include <SoftwareSerial.h>
SoftwareSerial gsm = SoftwareSerial(7,8);

byte GSM_phase = 0;

#define NUM_GSM_PHASES 24

enum _parseState {
  PS_DETECT_MSG_TYPE,

  PS_IGNORING_COMMAND_ECHO,
  PS_CSQ,
  PS_CREG_0,
  PS_CREG_1
};

byte parseState = PS_DETECT_MSG_TYPE;
char buffer[10];
byte pos = 0;

enum _actionState {
  AS_IDLE,
  AS_WAITING_FOR_RESPONSE
};

void resetBuffer() {
  memset(buffer, 0, sizeof(buffer));
  pos = 0;
}

void parseATText(byte b) {

  buffer[pos++] = b;

  Serial.write(b);

  if ( pos >= sizeof(buffer) )
    resetBuffer(); // just to be safe 

  switch (parseState) {
  case PS_DETECT_MSG_TYPE:
    {
      if ( b == '\n' )
        resetBuffer();
      else {       
        if ( pos == 3 && strcmp(buffer, "AT+") == 0 ) {
          parseState = PS_IGNORING_COMMAND_ECHO;
        }
        else if ( b == ':' ) {
          if ( strcmp(buffer, "+CSQ:") == 0 ) {
            parseState = PS_CSQ;
          }
          else if ( strcmp(buffer, "+CREG:") == 0 ) {
            parseState = PS_CREG_0;
          }
          resetBuffer();
        }
      }
    }
    break;

  case PS_IGNORING_COMMAND_ECHO:
    {
      if ( b == '\n' ) {
        parseState = PS_DETECT_MSG_TYPE;
        resetBuffer();
      }
    }
    break;

  case PS_CSQ:
    {
      if ( b == ',' ) {
        Serial.print("Got CSQ: ");
        GSM_signalQuality = atoi(buffer);
        Serial.println(GSM_signalQuality);
        parseState = PS_DETECT_MSG_TYPE;
        resetBuffer();
      }
    }
    break;

  case PS_CREG_0:
    {
      if ( b == ',' ) {
        parseState = PS_CREG_1;
        resetBuffer();
      }
    }
    break;

  case PS_CREG_1:
    {
      if ( b == '\n' ) {
        Serial.println(buffer);
        Serial.print("Got CREG: ");
        GSM_registered = atoi(buffer);
        Serial.println(GSM_registered);
        parseState = PS_DETECT_MSG_TYPE;
        resetBuffer();
      }
    }
    break;
  }
}

void GSM_flush() {
  while(gsm.available()) {
    parseATText(gsm.read());
  }
}

/*
void sendGSM_partial(const char* msg) {
  Serial.print(msg);
  gsm.print(msg);
  GSM_flush();
}

const char AT_SAPBR_0[] PROGMEM = "AT+SAPBR=3,1,\"APN\",\"vodafone\"";
const char AT_SAPBR_1[] PROGMEM = "AT+SAPBR=1,1";

const char AT_HTTPINIT[] PROGMEM = "AT+HTTPINIT";
const char AT_HTTPPARA_CID[] PROGMEM = "AT+HTTPPARA=\"CID\",1";
const char AT_HTTPPARA_URL[] PROGMEM = "AT+HTTPPARA=\"URL\",\"http://www.google.com\"";
const char AT_HTTPACTION[] PROGMEM = "AT+HTTPACTION=0";

const char AT_CSQ[] PROGMEM = "AT+CSQ";
*/
void setup() {

  Serial.begin(115200);
  gsm.begin(9600);

  Serial.println("Starting...");

  gsm.println(F("AT+SAPBR=3,1,\"APN\",\"vodafone\""));
  delay(500);
  gsm.println(F("AT+SAPBR=1,1")); // takes a while to finish, so...

  GSM_phase = 5;    // ... force a full round of waiting before doing anything else 
}

void GSM_loop() {

  GSM_phase = (GSM_phase + 1) % NUM_GSM_PHASES;

  Serial.println(GSM_phase);

  // This function is called approximately once per second, with 24 phases the phase will be
  // zero about once every 24 seconds
  switch ( GSM_phase ) {
    case 0: gsm.println(F("AT+CSQ")); break;
    case 1: gsm.println(F("AT+CREG?")); break;
    case 2: gsm.println(F("AT+HTTPINIT")); break;
    case 3: gsm.println(F("AT+HTTPPARA=\"CID\",1")); break;
    case 4: gsm.println(F("AT+HTTPPARA=\"URL\",\"http://elmo.thetamusic.com/tmpchris.php?s=gsm\"")); break;
    //case 4: gsm.println(F("AT+HTTPPARA=\"URL\",\"http://www.google.com\"")); break;
    case 5: gsm.println(F("AT+HTTPACTION=0")); break;

    // that should leave about 20 seconds for the HTTPACTION to complete
  } 

}

void loop() {
  GSM_flush();
  static unsigned long lastLoopTime = 0;
  unsigned long now = millis();
  if ( now - lastLoopTime > 1000 ) {
    GSM_loop();
    lastLoopTime = millis();
  }
}
