#include <Wire.h>

#define DO_SERIAL

#define CALIBRATION_COUNT       1000
#define MPU6050_ADDRESS         0x68
#define MPU6050_PWR_MGMT_1      0x6B   // R/W
#define ACCEL_VALUES_REGISTER   0x3B
#define GYRO_HIGH_BYTE_FIRST 

int accel[3];
int temp;

void i2c_write_reg(int address, byte reg, byte val) {
  Wire.beginTransmission(address);
  Wire.write(reg);
  Wire.write(val);
  Wire.endTransmission(true);
}

// Requires open I2C transmission with two bytes available
int value_read_int() {
  byte highByte = Wire.read();
  byte lowByte = Wire.read();
  return (highByte << 8) | lowByte;
}

void values_read_raw(){

  Wire.beginTransmission(MPU6050_ADDRESS);
  Wire.write(ACCEL_VALUES_REGISTER);
  Wire.endTransmission();
  Wire.requestFrom(MPU6050_ADDRESS, 8);

  while (Wire.available() < 8);

  for ( byte i = 0; i < 3; i++ )
    accel[i] = value_read_int();
  temp = value_read_int();

}

void setup() {
#ifdef DO_SERIAL
  Serial.begin(115200);
#endif
  
  i2c_write_reg(MPU6050_ADDRESS, 0x6B, 0x80);                     //PWR_MGMT_1    -- DEVICE_RESET 1
  delayMicroseconds(50000);
  i2c_write_reg(MPU6050_ADDRESS, 0x6B, 0x03);                     //PWR_MGMT_1    -- SLEEP 0; CYCLE 0; TEMP_DIS 0; CLKSEL 3 (PLL with Z Gyro reference)
  i2c_write_reg(MPU6050_ADDRESS, 0x1A, 0);                        //CONFIG        -- EXT_SYNC_SET 0 (disable input pin for data sync) ; default DLPF_CFG = 0 => ACC bandwidth = 260Hz  GYRO bandwidth = 256Hz)
  i2c_write_reg(MPU6050_ADDRESS, 0x1B, 0x08);                     //GYRO_CONFIG   -- FS_SEL = 3: Full scale set to 500 deg/sec
  
  // Clear the 'sleep' bit to start the sensor.
  MPU6050_write_reg (MPU6050_PWR_MGMT_1, 0);
  
}

void loop() {

  //Let's get the current gyro data and scale it to degrees per second for the pid calculations.
  values_read_raw();
  
#ifdef DO_SERIAL
  Serial.print( (temp/340.0f)+35 );
  Serial.print("\t");
  Serial.print(accel[0]/16384.0f);
  Serial.print("\t");
  Serial.print(accel[1]/16384.0f);
  Serial.print("\t");
  Serial.print(accel[2]/16384.0f);
  Serial.println("\t");
#endif

  delay(100);
}



int MPU6050_write(int start, const uint8_t *pData, int size)
{
  int n, error;

  Wire.beginTransmission(MPU6050_ADDRESS);
  n = Wire.write(start);        // write the start address
  if (n != 1)
    return (-20);

  n = Wire.write(pData, size);  // write data bytes
  if (n != size)
    return (-21);

  error = Wire.endTransmission(true); // release the I2C-bus
  if (error != 0)
    return (error);

  return (0);         // return : no error
}

int MPU6050_write_reg(int reg, uint8_t data)
{
  return MPU6050_write(reg, &data, 1);
}
