#include <Wire.h>

#define DO_SERIAL

#define MS5611_ADDRESS 0x77

unsigned long i2cErrorCount = 0;
#ifdef DO_SERIAL
unsigned int calibrationData[6];
#endif

void logError(char* str) {
#ifdef DO_SERIAL
  Serial.println(str);
#else
  i2cErrorCount++;
#endif
}

void twiSendCommand(byte address, byte command)
{
  Wire.beginTransmission(address);
  
  if (!Wire.write(command))
#ifdef DO_SERIAL
    Serial.println("Wire: error doing write()");
#else
    i2cErrorCount++;
#endif

  if (Wire.endTransmission()) {
#ifdef DO_SERIAL
    Serial.print("Wire: error when sending command: ");
    Serial.println(command, HEX);
#else
    i2cErrorCount++;
#endif
  }
}

void twiGetValue(byte command, byte numBytes, byte* dest)
{
  twiSendCommand(MS5611_ADDRESS, command);
  Wire.requestFrom(MS5611_ADDRESS, numBytes);
  if ( Wire.available() != numBytes ) 
    logError("Wire: available() gave unexpected result");
  byte index = numBytes - 1;
  for (byte i = 0; i < numBytes; i++)
    dest[index--] = Wire.read();
}

void getMS5611Data(uint32_t& D1, uint32_t& D2)
{  
  D1 = D2 = 0;

  twiSendCommand(MS5611_ADDRESS, 0x48); // start pressure conversion
  delay(10);
  twiGetValue(0x00, 3, (byte*)&D1);
  
  twiSendCommand(MS5611_ADDRESS, 0x50); // start temperature conversion
  delay(1);
  twiGetValue(0x00, 3, (byte*)&D2);
}

void setup()
{
  Wire.begin();

  twiSendCommand(MS5611_ADDRESS, 0x1E); // reset
  delay(100);
  
#ifdef DO_SERIAL
  Serial.begin(115200);

  for ( byte i = 0; i < 6; i++) {
    byte cmd = 0xA0 + (i+1) * 2; // calibration data
    twiGetValue(cmd, 2, (byte*)&calibrationData[i]);
  }

  for (byte i = 0; i < 6; i++) {
    Serial.print(i);
    Serial.print(" calib: ");
    Serial.println(calibrationData[i]);
  }
#endif
}

void loop()
{
  uint32_t D1, D2;
  getMS5611Data(D1, D2);
  
#ifdef DO_SERIAL

  // names as given on page 8 of datasheet
  int32_t dT;
  int64_t OFF, SENS;
  long P, TEMP;

  dT = D2 - ((long)calibrationData[4] << 8);
  TEMP = 2000 + (((int64_t)dT * (int64_t)calibrationData[5]) >> 23);
  OFF = ((unsigned long)calibrationData[1] << 16) + (((int64_t)calibrationData[3] * dT) >> 7);
  SENS = ((unsigned long)calibrationData[0] << 15) + (((int64_t)calibrationData[2] * dT) >> 8);
  P = ((D1 * (SENS >> 21)) - OFF) >> 15;

  Serial.print((float)P/100.0f);
  Serial.print(" mbar, ");
  Serial.print((float)TEMP/100.0f);
  Serial.println(" degC");  
#endif  

  //delay(1000);
}
