
uint32_t MS5611_D1, MS5611_D2;

//--------------------------------------------------

#define MS5611_ADDRESS 0x77

#ifdef DO_SERIAL
unsigned int MS5611_calibrationData[6];
#endif

void getMS5611Data(uint32_t& D1, uint32_t& D2)
{  
  D1 = D2 = 0;

  twiWriteByte(MS5611_ADDRESS, 0x48); // start pressure conversion
  delay(10);
  twiReadData(MS5611_ADDRESS, 0x00, (byte*)&D1, 3);
  
  twiWriteByte(MS5611_ADDRESS, 0x50); // start temperature conversion
  delay(1);
  twiReadData(MS5611_ADDRESS, 0x00, (byte*)&D2, 3);
}

void MS5611_setup()
{
  twiWriteByte(MS5611_ADDRESS, 0x1E); // reset
  delay(100);
  
#ifdef DO_SERIAL
  for ( byte i = 0; i < 6; i++) {
    byte cmd = 0xA0 + (i+1) * 2; // calibration data
    twiReadData(MS5611_ADDRESS, cmd, (byte*)&MS5611_calibrationData[i], 2);
  }

  for (byte i = 0; i < 6; i++) {
    Serial.print(i);
    Serial.print(" MS5611 calib: ");
    Serial.println(MS5611_calibrationData[i]);
  }
#endif
}

void MS5611_loop()
{
  getMS5611Data(MS5611_D1, MS5611_D2);
  
#ifdef DO_SERIAL

  // names as given on page 8 of datasheet
  int32_t dT;
  int64_t OFF, SENS;
  long P, TEMP;

  dT = MS5611_D2 - ((long)MS5611_calibrationData[4] << 8);
  TEMP = 2000 + (((int64_t)dT * (int64_t)MS5611_calibrationData[5]) >> 23);
  OFF = ((unsigned long)MS5611_calibrationData[1] << 16) + (((int64_t)MS5611_calibrationData[3] * dT) >> 7);
  SENS = ((unsigned long)MS5611_calibrationData[0] << 15) + (((int64_t)MS5611_calibrationData[2] * dT) >> 8);
  P = ((MS5611_D1 * (SENS >> 21)) - OFF) >> 15;

  Serial.print("MS5611: ");
  Serial.print((float)TEMP/100.0f);
  Serial.print(" degC, ");
  Serial.print((float)P/100.0f);
  Serial.println(" mbar"); 
#endif  
}

